@@ -524,6 +524,13 @@ static bool match_interface_instance(const void *a, const void *b)
 	return false;
 }
 
+static bool match_interface_instance_ptr(const void *a, const void *b)
+{
+	const struct interface_instance *instance = a;
+
+	return instance->interface == b;
+}
+
 static void interface_add_record_free(void *data)
 {
 	struct interface_add_record *rec = data;
@@ -1223,10 +1230,12 @@ bool _dbus_object_tree_property_changed(struct l_dbus *dbus,
 	return true;
 }
 
-static void set_property_complete(struct l_dbus *dbus,
+static void pending_property_set_done(struct l_dbus *dbus,
 					struct l_dbus_message *message,
 					struct l_dbus_message *reply)
 {
+	const char *member;
+	const char *interface_name;
 	const char *property_name;
 	struct l_dbus_message_iter variant;
 
@@ -1237,16 +1246,24 @@ static void set_property_complete(struct l_dbus *dbus,
 
 	l_dbus_send(dbus, l_dbus_message_ref(reply));
 
-	if (!l_dbus_message_is_error(reply)) {
-		l_dbus_message_get_arguments(message, "sv", &property_name,
-						&variant);
+	member = l_dbus_message_get_member(message);
+	if (!strcmp(member, "SetProperty")) {
+		if (!l_dbus_message_get_arguments(message, "sv",
+						&property_name, &variant))
+			goto done;
 
-		_dbus_object_tree_property_changed(dbus,
+		interface_name = l_dbus_message_get_interface(message);
+	} else if (strcmp(member, "Set") ||
+			!l_dbus_message_get_arguments(message, "ssv",
+							&interface_name,
+							&property_name,
+							&variant))
+		goto done;
+
+	_dbus_object_tree_property_changed(dbus,
 					l_dbus_message_get_path(message),
-					l_dbus_message_get_interface(message),
-					property_name);
-	}
-
+					interface_name, property_name);
+done:
 	l_dbus_message_unref(message);
 	l_dbus_message_unref(reply);
 }
@@ -1289,10 +1306,10 @@ static struct l_dbus_message *old_set_property(struct l_dbus *dbus,
 						property_name);
 
 	reply = property->setter(dbus, l_dbus_message_ref(message), &variant,
-					set_property_complete, user_data);
+					pending_property_set_done, user_data);
 
 	if (reply)
-		set_property_complete(dbus, message, reply);
+		pending_property_set_done(dbus, message, reply);
 
 	return NULL;
 }
@@ -1489,8 +1506,7 @@ bool _dbus_object_tree_add_interface(struct _dbus_object_tree *tree,
 	 * Check to make sure we do not have this interface already
 	 * registered for this object
 	 */
-	if (l_queue_find(object->instances, match_interface_instance,
-				(char *) interface))
+	if (l_queue_find(object->instances, match_interface_instance_ptr, dbi))
 		return false;
 
 	instance = l_new(struct interface_instance, 1);
@@ -1546,6 +1562,25 @@ bool _dbus_object_tree_add_interface(struct _dbus_object_tree *tree,
 	return true;
 }
 
+void *_dbus_object_tree_get_interface_data(struct _dbus_object_tree *tree,
+						const char *path,
+						const char *interface)
+{
+	struct object_node *object;
+	struct interface_instance *instance;
+
+	object = l_hashmap_lookup(tree->objects, path);
+	if (!object)
+		return NULL;
+
+	instance = l_queue_find(object->instances, match_interface_instance,
+				(char *) interface);
+	if (!instance)
+		return NULL;
+
+	return instance->user_data;
+}
+
 static bool match_object_manager_path(const void *a, const void *b)
 {
 	const struct object_manager *manager = a;
@@ -1815,33 +1850,6 @@ static struct l_dbus_message *properties_get(struct l_dbus *dbus,
 	return reply;
 }
 
-static void properties_set_complete(struct l_dbus *dbus,
-					struct l_dbus_message *message,
-					struct l_dbus_message *reply)
-{
-	const char *interface_name, *property_name;
-	struct l_dbus_message_iter variant;
-
-	if (!reply) {
-		reply = l_dbus_message_new_method_return(message);
-		l_dbus_message_set_arguments(reply, "");
-	}
-
-	l_dbus_send(dbus, l_dbus_message_ref(reply));
-
-	if (!l_dbus_message_is_error(reply)) {
-		l_dbus_message_get_arguments(message, "ssv", &interface_name,
-						&property_name, &variant);
-
-		_dbus_object_tree_property_changed(dbus,
-					l_dbus_message_get_path(message),
-					interface_name, property_name);
-	}
-
-	l_dbus_message_unref(message);
-	l_dbus_message_unref(reply);
-}
-
 static struct l_dbus_message *properties_set(struct l_dbus *dbus,
 						struct l_dbus_message *message,
 						void *user_data)
@@ -1890,8 +1898,7 @@ static struct l_dbus_message *properties_set(struct l_dbus *dbus,
 	/* If we got here the object must exist */
 
 	instance = l_queue_find(object->instances,
-				match_interface_instance,
-				(char *) interface_name);
+				match_interface_instance_ptr, interface);
 	if (!instance)
 		return l_dbus_message_new_error(message,
 						"org.freedesktop.DBus.Error."
@@ -1900,11 +1907,11 @@ static struct l_dbus_message *properties_set(struct l_dbus *dbus,
 						interface_name);
 
 	reply = property->setter(dbus, l_dbus_message_ref(message), &variant,
-					properties_set_complete,
+					pending_property_set_done,
 					instance->user_data);
 
 	if (reply)
-		properties_set_complete(dbus, message, reply);
+		pending_property_set_done(dbus, message, reply);
 
 	return NULL;
 }
